@@ -8,18 +8,7 @@ from dataclasses import dataclass, field
 from itertools import chain
 from pathlib import Path
 from textwrap import dedent, indent, wrap
-from typing import (
-    Any,
-    Dict,
-    Generator,
-    Iterable,
-    Iterator,
-    List,
-    Optional,
-    Set,
-    Tuple,
-    Union,
-)
+from typing import Any, DefaultDict, Generator, Iterable, Iterator
 
 import black
 import isort.api
@@ -43,10 +32,10 @@ except ImportError:
     from xmlschema import qnames
 
 
-# Track all camel-to-snake and pluralization results so we can include them in the model.
-camel_snake_registry: Dict[str, str] = {}
-plural_registry: Dict[Tuple[str, str], str] = {}
-
+# Track all camel-to-snake and pluralization results so we can include them in the model
+camel_snake_registry: dict[str, str] = {}
+plural_registry: dict[tuple[str, str], str] = {}
+LISTS: DefaultDict[str, set[str]] = DefaultDict(set)
 
 # FIXME: Work out a better way to implement these override hacks.
 
@@ -54,9 +43,9 @@ plural_registry: Dict[Tuple[str, str], str] = {}
 @dataclass
 class Override:
     type_: str
-    default: Optional[str] = None
-    imports: Optional[str] = None
-    body: Optional[str] = None
+    default: str | None = None
+    imports: str | None = None
+    body: str | None = None
 
     def __post_init__(self) -> None:
         if self.imports:
@@ -67,11 +56,11 @@ class Override:
 
 @dataclass
 class ClassOverride:
-    base_type: Optional[str] = None
-    imports: Optional[str] = None
-    fields: Optional[str] = None
-    fields_suppress: Set[str] = field(default_factory=set)
-    body: Optional[str] = None
+    base_type: str | None = None
+    imports: str | None = None
+    fields: str | None = None
+    fields_suppress: set[str] = field(default_factory=set)
+    body: str | None = None
 
     def __post_init__(self) -> None:
         if self.imports:
@@ -248,16 +237,16 @@ CLASS_OVERRIDES = {
 
             @classmethod
             def from_xml(cls, xml: Union[Path, str]) -> 'OME':
-                from ome_types._convenience import from_xml
+                from ome_types import from_xml
                 return from_xml(xml)
 
             @classmethod
             def from_tiff(cls, path: Union[Path, str]) -> 'OME':
-                from ome_types._convenience import from_tiff
+                from ome_types import from_tiff
                 return from_tiff(path)
 
             def to_xml(self) -> str:
-                from ome_types.schema import to_xml
+                from ome_types import to_xml
                 return to_xml(self)
         """,
     ),
@@ -428,11 +417,12 @@ def local_import(item_type: str) -> str:
     return f"from .{camel_to_snake(item_type)} import {item_type}"
 
 
-def get_docstring(
-    component: Union[XsdComponent, XsdType], summary: bool = False
-) -> str:
+def get_docstring(component: XsdComponent | XsdType, summary: bool = False) -> str:
     try:
         doc = dedent(component.annotation.documentation[0].text).strip()
+        # some docstrings include a start ('*Word') which makes sphinx angry
+        # this line wraps those in backticks
+        doc = re.sub(r"(\*\w+)\s", r"`\1` ", doc)
         # make sure the first line is followed by a double newline
         # and preserve paragraphs
         if summary:
@@ -445,7 +435,7 @@ def get_docstring(
         return ""
 
 
-def make_dataclass(component: Union[XsdComponent, XsdType]) -> List[str]:
+def make_dataclass(component: XsdComponent | XsdType) -> list[str]:
     class_override = CLASS_OVERRIDES.get(component.local_name, None)
     lines = ["from ome_types._base_type import OMEType", ""]
     if isinstance(component, XsdType):
@@ -505,7 +495,7 @@ def make_dataclass(component: Union[XsdComponent, XsdType]) -> List[str]:
     return lines
 
 
-def make_abstract_class(component: XsdComponent) -> List[str]:
+def make_abstract_class(component: XsdComponent) -> list[str]:
     # FIXME: ? this might be a bit of an OME-schema-specific hack
     # this seems to be how abstract is used in the OME schema
     for e in component.iter_components():
@@ -540,7 +530,7 @@ def make_abstract_class(component: XsdComponent) -> List[str]:
     return lines
 
 
-def make_enum(component: XsdComponent) -> List[str]:
+def make_enum(component: XsdComponent) -> list[str]:
     name = component.local_name
     _type = component.type if hasattr(component, "type") else component
     if _type.is_list():
@@ -555,7 +545,7 @@ def make_enum(component: XsdComponent) -> List[str]:
         lines += indent(doc, "    ").splitlines()
     enum_elems = list(_type.elem.iter("enum"))
     facets = _type.get_facet(qnames.XSD_ENUMERATION)
-    members: List[Tuple[str, str]] = []
+    members: list[tuple[str, str]] = []
     if enum_elems:
         for el, value in zip(enum_elems, facets.enumeration):
             _name = el.attrib["enum"]
@@ -571,7 +561,7 @@ def make_enum(component: XsdComponent) -> List[str]:
     return lines
 
 
-def make_color() -> List[str]:
+def make_color() -> list[str]:
     color = """
     from pydantic import color
     class Color(color.Color):
@@ -616,7 +606,7 @@ facet_converters = {
 
 def iter_all_members(
     component: XsdComponent,
-) -> Generator[Union[XsdElement, XsdAttribute], None, None]:
+) -> Generator[XsdElement | XsdAttribute, None, None]:
     for c in component.iter_components((XsdElement, XsdAttribute)):
         if c is component:
             continue
@@ -624,14 +614,13 @@ def iter_all_members(
 
 
 def iter_members(
-    component: Union[XsdElement, XsdType]
-) -> Generator[Union[XsdElement, XsdAttribute], None, None]:
+    component: XsdElement | XsdType,
+) -> Generator[XsdElement | XsdAttribute, None, None]:
     if isinstance(component, XsdElement):
         for attr in component.attributes.values():
             if isinstance(attr, XsdAttribute):
                 yield attr
-        for elem in component.iterchildren():
-            yield elem
+        yield from component.iterchildren()
     else:
         yield from iter_all_members(component)
 
@@ -642,9 +631,10 @@ def is_enum_type(obj: XsdType) -> bool:
 
 
 class Member:
-    def __init__(self, component: Union[XsdElement, XsdAttribute]):
+    def __init__(self, component: XsdElement | XsdAttribute):
         self.component = component
-        assert not component.is_global()
+        if component.is_global():
+            raise ValueError(f"global component {component!r} not allowed")
 
     @property
     def identifier(self) -> str:
@@ -660,7 +650,7 @@ class Member:
         return name
 
     @property
-    def plural(self) -> Optional[str]:
+    def plural(self) -> str | None:
         """Plural form of component name, if available."""
         if (
             isinstance(self.component, XsdElement)
@@ -669,7 +659,8 @@ class Member:
             and self.component.ref.annotation
         ):
             appinfo = self.component.ref.annotation.appinfo
-            assert len(appinfo) == 1, "unexpected multiple appinfo elements"
+            if len(appinfo) != 1:
+                raise ValueError("unexpected multiple appinfo elements")
             plural = appinfo[0].find("xsdfu/plural")
             if plural is not None:
                 return plural.text
@@ -736,14 +727,14 @@ class Member:
             return self.component.local_name
         return name
 
-    def locals(self) -> List[str]:
+    def locals(self) -> list[str]:
         if self.key in OVERRIDES:
             return []
         if isinstance(self.component, (XsdAnyElement, XsdAnyAttribute)):
             return []
         if not self.type or self.type.is_global():
             return []
-        locals_: List[str] = []
+        locals_: list[str] = []
         # FIXME: this bit is mostly hacks
         if self.type.is_complex() and self.component.ref is None:
             locals_.append("\n".join(make_dataclass(self.component)) + "\n")
@@ -753,7 +744,7 @@ class Member:
             locals_.append("\n".join(make_enum(self.component)) + "\n")
         return locals_
 
-    def imports(self) -> List[str]:
+    def imports(self) -> list[str]:
         if self.key in OVERRIDES:
             _imp = OVERRIDES[self.key].imports
             return [_imp] if _imp else []
@@ -797,7 +788,8 @@ class Member:
         if isinstance(self.component, (XsdAnyElement, XsdAnyAttribute)):
             return "Any"
         if self.component.ref is not None:
-            assert self.component.ref.is_global()
+            if not self.component.ref.is_global():
+                raise ValueError("local ref not supported")
             return self.component.ref.local_name
 
         if self.type.is_datetime():
@@ -821,13 +813,15 @@ class Member:
 
     @property
     def full_type_string(self) -> str:
-        """full type, like Optional[List[str]]"""
+        """full type, like Optional[List[str]]."""
         if self.key in OVERRIDES and self.type_string:
             return f": {self.type_string}"
         type_string = self.type_string
         if not type_string:
             return ""
         if not self.max_occurs:
+            LISTS[self.parent_name].add(self.component.local_name)
+
             type_string = f"List[{type_string}]"
         elif self.is_optional:
             type_string = f"Optional[{type_string}]"
@@ -850,14 +844,16 @@ class Member:
                     default_val = f"{self.type_string}('{default_val}')"
                 elif hasattr(builtins, self.type_string):
                     default_val = repr(getattr(builtins, self.type_string)(default_val))
-                if self.is_decimal:
+                if self.type_string == "Color":
+                    default_val = "Color('white')"
+                elif self.is_decimal:
                     default_val = f"cast({self.type_string}, {default_val})"
             else:
                 default_val = "None"
         return f" = {default_val}"
 
     @property
-    def max_occurs(self) -> Optional[int]:
+    def max_occurs(self) -> int | None:
         default = None if self.type.is_list() else 1
         return getattr(self.component, "max_occurs", default)
 
@@ -882,7 +878,7 @@ class Member:
 class MemberSet:
     def __init__(self, initial: Iterable[Member] = ()):
         # Use a list to maintain insertion order.
-        self._members: List[Member] = []
+        self._members: list[Member] = []
         self.update(initial)
 
     def add(self, member: Member) -> None:
@@ -897,7 +893,7 @@ class MemberSet:
         for member in members:
             self.add(member)
 
-    def lines(self, indent: int = 1) -> List[str]:
+    def lines(self, indent: int = 1) -> list[str]:
         if not self._members:
             lines = ["    " * indent + "pass"]
         else:
@@ -907,13 +903,13 @@ class MemberSet:
             ]
         return lines
 
-    def imports(self) -> List[str]:
+    def imports(self) -> list[str]:
         return list(chain.from_iterable(m.imports() for m in self._members))
 
-    def locals(self) -> List[str]:
+    def locals(self) -> list[str]:
         return list(chain.from_iterable(m.locals() for m in self._members))
 
-    def body(self) -> List[str]:
+    def body(self) -> list[str]:
         return [m.body() for m in self._members]
 
     def has_non_default_args(self) -> bool:
@@ -938,8 +934,9 @@ class MemberSet:
 
 
 class GlobalElem:
-    def __init__(self, elem: Union[XsdElement, XsdType]):
-        assert elem.is_global()
+    def __init__(self, elem: XsdElement | XsdType):
+        if not elem.is_global():
+            raise ValueError("Element must be global")
         self.elem = elem
 
     @property
@@ -968,7 +965,7 @@ class GlobalElem:
                 raise NotImplementedError("Unexpected enum with multiple facets")
         return is_enum
 
-    def _simple_class(self) -> List[str]:
+    def _simple_class(self) -> list[str]:
         if self.type.local_name == "Color":
             return make_color()
         if self.is_enum:
@@ -979,8 +976,8 @@ class GlobalElem:
         if self.type.local_name == "Hex40":
             return [
                 "class Hex40(ConstrainedStr):",
-                "    min_length = 20",
-                "    max_length = 20",
+                "    min_length = 40",
+                "    max_length = 40",
             ]
 
         lines = []
@@ -998,7 +995,7 @@ class GlobalElem:
             members.extend([f"    {line}" for line in facet_converters[key](facet)])
         lines.extend(members if members else ["    pass"])
         if any("re.compile" in m for m in members):
-            lines = ["import re", ""] + lines
+            lines = ["import re", "", *lines]
         return lines
 
     def lines(self) -> str:
@@ -1027,6 +1024,30 @@ _this_dir = os.path.dirname(__file__)
 _url = os.path.join(_this_dir, "ome_types", "ome-2016-06.xsd")
 _target = os.path.join(_this_dir, "ome_types", "model")
 
+AnnotationID_Override = r"""class AnnotationID(LSID):
+    regex = re.compile(
+        r"(urn:lsid:([\w\-\.]+\.[\w\-\.]+)+:Annotation:\S+)|(Annotation:\S+)"
+    )
+
+    @classmethod
+    def __get_validators__(cls):
+        yield cls.validate
+
+    @classmethod
+    def validate(cls, v) -> "AnnotationID":
+        if not cls.regex.match(v):
+            search = cls.regex.search(v)
+            if search:
+                import warnings
+
+                new_v = search.group()
+                warnings.warn(f"Casting invalid AnnotationID {v!r} to {new_v!r}")
+                v = new_v
+        return v
+"""
+
+_SIMPLE_OVERRIDES = {"AnnotationID": AnnotationID_Override}
+
 
 def convert_schema(url: str = _url, target_dir: str = _target) -> None:
     print("Inspecting XML schema ...")
@@ -1036,7 +1057,7 @@ def convert_schema(url: str = _url, target_dir: str = _target) -> None:
     print("Building model ...")
     shutil.rmtree(target_dir, ignore_errors=True)
     init_imports = []
-    simples: List[GlobalElem] = []
+    simples: list[GlobalElem] = []
     for elem in sorted(schema.types.values(), key=sort_types):
         if elem.local_name in OVERRIDES:
             continue
@@ -1056,13 +1077,15 @@ def convert_schema(url: str = _url, target_dir: str = _target) -> None:
         init_imports.append((converter.fname, elem.local_name))
         converter.write(filename=targetfile)
 
-    text = "\n".join([s.format() for s in simples])
+    text = "\n".join(
+        [_SIMPLE_OVERRIDES.get(s.elem.local_name) or s.format() for s in simples]
+    )
     text = black_format(sort_imports(text))
     with open(os.path.join(target_dir, "simple_types.py"), "w", encoding="utf-8") as f:
         f.write(text)
 
     text = ""
-    for fname, classname in init_imports:
+    for _fname, classname in init_imports:
         text += local_import(classname) + "\n"
     text = sort_imports(text)
     text += f"\n\n__all__ = [{', '.join(sorted(repr(i[1]) for i in init_imports))}]"
@@ -1085,6 +1108,7 @@ def convert_schema(url: str = _url, target_dir: str = _target) -> None:
     text += "\n\n_snake_to_camel = " + repr(
         {camel_snake_registry[k]: k for k in sorted(camel_snake_registry)}
     )
+    text += "\n\n_lists = " + repr(dict(LISTS))
     text = black_format(text)
     with open(os.path.join(target_dir, "__init__.py"), "w", encoding="utf-8") as f:
         f.write(text)
